/*
   $Id: UIpi.c,v 1.3 2001/02/11 17:35:58 yossa Exp $
   Author: Micha Moskal <malekith@pld.org.pl>
   Based on UInewt.c by Pawe Koodziej <pawelk@pld.org.pl>
 */

#include "config.h"
#include <stdio.h>
#include <stdlib.h>
#include <rpm/rpmlib.h>
#include <dirent.h>
#include <unistd.h>
#include <fcntl.h>
#include <string.h>
#include <sys/stat.h>
#include <errno.h>

#include <pi/pi.h>

#include "rpmmen.h"
#include "rpmmen_config.h"
#include "rpmpldi.h"
#include "module.h"

extern char *pkgsListFname;

#define REDIRECT_STDOUT redirect_out(1)
#define RESTORE_STDOUT  redirect_out(0)

void pi_selector();

#if 0
#define select_from_file pldi_select_from_file
#define rpmmen_install pldi_rpmmen_install
#define rpmmen_scandir pldi_rpmmen_scandir
#endif

struct install_ui_tag {
	struct pi_object 
		*form,
		*pkg_label, *pkg_pbar,
		*pkgs_label, *pkgs_pbar;
	int npkgs;
	int ninstpkgs;
	unsigned long nbytes, ninstbytes;
};

static struct install_ui_tag install_ui;
static const char *stdoutfile = RPMMEN_STDOUT_FILE;
static int excludeDoc;	/* ustawione jeli instalujemy bez dokumentacji */


static void make_progress_window()
{
	struct install_ui_tag *self;
	int win_width;

	memset(&install_ui, 0, sizeof(install_ui));
	self=&install_ui;

	win_width = pi_screen_width - 20;
	
	self->form = pi_new_window(pi_add_object(0,
		pi_tag_w, win_width,
		pi_tag_h, 10,
		pi_tag_caption, _("Packages installation"),
		0), 0);
	pi_center(self->form);
	self->pkg_label = pi_add(self->form, label, (0,
			pi_tag_x, 2,
			pi_tag_y, 1,
			pi_tag_w, win_width - 4,
			pi_tag_caption, _("Preparing to installation..."),
		0));
	self->pkg_pbar = pi_add(self->form, progressbar, (0,
			pi_tag_x, 2,
			pi_tag_y, 3,
			pi_tag_w, win_width - 4,
		0));
	self->pkgs_label = pi_add(self->form, label, (0,
			pi_tag_x, 2,
			pi_tag_y, 5,
			pi_tag_w, win_width - 4,
		0));
	self->pkgs_pbar = pi_add(self->form, progressbar, (0,
			pi_tag_x, 2,
			pi_tag_y, 6,
			pi_tag_w, win_width - 4,
			pi_tag_max, self->nbytes = 
				rpmpldi_selected_size(&self->npkgs),
		0));
	pi_add_child(pi_root_object, self->form);
}

/* redirect_out - if( to >0  ) redirect stdout and stderr to file, if (!to)
 * redirect to console. if (to < 0)  redirect to console, and close file.
 */
static void redirect_out(int to)
{
	static int new = -1, o2 = -1, o1 = -1;

	if (new < 0) {
		new = open(stdoutfile, O_RDWR | O_CREAT, 0644);

		log_debug("redirect_out: file handle: %d", new);
		if (new == -1) {
			log_message(_("can't open file for stdout "
					"(not good, but not critical) : %s"),
					strerror(errno));
			return;
		}
	}
	if (o1 == -1) {
		o2 = dup(2);
		o1 = dup(1);
	}
	if (to > 0) {
		dup2(new, 2);
		dup2(new, 1);
	}
	if (to == 0) {
		dup2(o2, 2);
		dup2(o1, 1);
	}
	if (to < 0) {
		redirect_out(0);
		close(new);
		new = -1;
	}
}


/* zmie napisy w oknie z postpem instalacji na dane nowego pakietu */
static void UIOpenPackage(const TPkgInfo *pkg)
{
	static char label[100], label2[100];

	RESTORE_STDOUT;

	if (pkg == NULL)
		strncpy(label, _("Preparing to installation..."), sizeof(label));
	else 
		snprintf(label, sizeof(label), _("Installing package %s"), pkg->name);

	pi_set_tag(install_ui.pkg_label, pi_tag_caption, label);

	snprintf(label2, sizeof(label2), 
		_("%d of %d packages installed (%ldk / %ldk)"),
		install_ui.ninstpkgs, install_ui.npkgs,
		install_ui.ninstbytes / 1024, install_ui.nbytes / 1024);
	pi_set_tag(install_ui.pkgs_label, pi_tag_caption, label2);

	pi_draw_all();
	REDIRECT_STDOUT;
}

static void UIInstallProgress(const unsigned long amount, const unsigned long total)
{
	if (amount && amount == total) { /* last notification */
		install_ui.ninstbytes += total;
		pi_set_tag(install_ui.pkgs_pbar, pi_tag_pos, 
			   install_ui.ninstbytes);
	}

	pi_set_tag(install_ui.pkg_pbar, pi_tag_max, total);
	pi_set_tag(install_ui.pkg_pbar, pi_tag_pos, amount);
	if (pi_progressbar_need_update(install_ui.pkg_pbar)) {
		RESTORE_STDOUT;
		pi_draw_all();
		REDIRECT_STDOUT;
	}
}

static void UIClosePackage(const TPkgInfo *pkg)
{
	log_debug("Close package %s", pkg->name);
	install_ui.ninstpkgs++;
}

static void UIrpmmen_deps(int ac, int pos, int n_max, const char *slabel)
{
	static struct pi_object *win, *label, *pbar;
	int win_width;

	switch (ac) {
	case 0:
		RESTORE_STDOUT;
		if (win == 0) {
			win_width = pi_screen_width - 20;
			win = pi_new_window(pi_add_object(0,
				pi_tag_w, win_width,
				pi_tag_h, 6,
				pi_tag_caption, _("Adding packages "
						  "required by other"),
			0), 0);
			pi_center(win);
			label = pi_add(win, label, (0,
				pi_tag_x, 2,
				pi_tag_y, 1,
				pi_tag_w, win_width - 4,
			0));
			pbar = pi_add(win, progressbar, (0,
				pi_tag_x, 2,
				pi_tag_y, 3,
				pi_tag_w, win_width - 4,
			0));
			pi_add_child(pi_root_object, win);
		}
		REDIRECT_STDOUT;
	case 1:
		RESTORE_STDOUT;
		if (slabel)
			pi_set_tag(label, pi_tag_caption, slabel);
		if (n_max)
			pi_set_tag(pbar, pi_tag_max, n_max);
		pi_set_tag(pbar, pi_tag_pos, pos);
		if (pi_progressbar_need_update(pbar) || slabel) {
			pi_draw(win);
			pi_refresh();
		}
		REDIRECT_STDOUT;
		break;
	case 2:
		RESTORE_STDOUT;
		if (win) {
			pi_kill(win);
			pi_draw_all();
			label = pbar = win = 0;
		}
		break;
	}
}

/**
 * \return static buffer  !!!
 */
static char *mkstrNVRF(char *name,char *ver,char *rel,int flag)
{
	static char ret[100];

	strcpy(ret,name);
	if(ver&&strlen(ver)){
		strcat(ret," (");
		if(flag&RPMSENSE_LESS)
			strcat(ret,"<");
		if(flag&RPMSENSE_EQUAL)
			strcat(ret,"=");
		if(flag&RPMSENSE_GREATER)
			strcat(ret,">");
		strcat(ret,ver);
		if(rel){
			strcat(ret,"-");
			strcat(ret,rel);
		}
		strcat(ret,")");
	}
	return ret;
}

static void add_str(char **s1, char *s2)
{
	*s1 = realloc(*s1, strlen(*s1 ? *s1 : "") + strlen(s2) + 1);
	strcat(*s1, s2);
}

static int UIcb_conflict(int action,char *name1,char *ver1,char *rel1,
		char *name2,char *ver2,int flag2)
{
	static char  *ctext=NULL,*utext=NULL;
	char *uhead=_("Packages below requires some features which are\n"
			"not provided by any other package.\n");
	char *chead=_("Packages below can't be installed simultaneously.\n"
			"You have to manually deselect some of them.\n");

	switch (action) {
	case RPMMEN_CB_CONFLICT_ADD:
		if (ctext == 0)
			ctext = strdup(chead);

		add_str(&ctext, "\n");
		add_str(&ctext, mkstrNVRF(name1, ver1, rel1, 0));
		add_str(&ctext, "  <--->  ");
		add_str(&ctext, mkstrNVRF(name2, ver2, NULL, flag2));
		break;
		
	case RPMMEN_CB_CONFLICT_UNSATISFIED:
		if (utext == 0)
			utext = strdup(uhead);
		add_str(&utext, _("\npackage "));
		add_str(&utext, mkstrNVRF(name1, ver1, rel1, 0));
		add_str(&utext, _("  requires "));
		add_str(&utext, mkstrNVRF(name2, ver2, NULL, flag2));
		break;
		
	case RPMMEN_CB_CONFLICT_NEW:
		free(ctext);
		ctext = NULL;
		free(utext);
		utext=NULL;
		break;
		
	case RPMMEN_CB_CONFLICT_SHOW:
		if((ctext == 0) && (utext == 0))
			return 0;
		if (ctext && utext) {
			add_str(&ctext, "\n\n");
			add_str(&ctext, utext);
			free(utext);
		} else 
			ctext = utext;
		utext = 0;
		pi_msgbox(pi_add_object(0,
			pi_tag_title, _("Dependencies problems"),
			pi_tag_caption, ctext,
			pi_tag_button_ok, _("_OK"),
			0));
		free(ctext);
		ctext = 0;
		return 1;
	}
	return 0; /* if function return heare returned value is _not_ 
		     important */
}

static void status_msg(const char *msg)
{
	static struct pi_object *win, *label;

	if (win == 0) {
		if (msg == 0)
			return;
			
		win = pi_new_window(pi_add_object(0,
				pi_tag_w, 60,
				pi_tag_h, 4,
				pi_tag_caption, _("Status"),
			0), 0);

		pi_center(win);
		
		label = pi_add(win, label, (0,
				pi_tag_x, 1,
				pi_tag_y, 1,
				pi_tag_w, 58,
			0));
		pi_add_child(pi_root_object, win);
	}

	if (msg) {
		pi_set_tag(label, pi_tag_caption, msg);
		pi_draw_all();
	} else {
		pi_kill(win);
		win = label = 0;
	}
}

/** run rpms instalation (with user interface).

  \param names -- names of packages which should be preselected
  \param count --count of names
  \return -1 - ok - instalaction aborted by user <br>
  \return 0   - ok - rpms installation successful <br>
  \return  1  - destantion media error <br>
  \return 2  - source media error 

 */
static int rpmmenUI_run(char *src, char *dest, const char *stdoutf,
		char **names,int count)
{
	int i, err = 0;
	struct rpmmen_cb cb;
	int again = 0;
	char message[1000];
	char *error_msg = 0;

	if (stdoutf)
		stdoutfile = stdoutf;

	/* rpmlib prints all instalation's errors on stderr. So now i create
	 * file to which it will be redirected
	 */
	status_msg(_("Building list of available packages..."));


	cb.pkg_open = UIOpenPackage;
	cb.pkg_inst_progress = UIInstallProgress;
	cb.pkg_close = UIClosePackage;
	cb.deps_prog = UIrpmmen_deps;
	cb.conflict  =UIcb_conflict;

	REDIRECT_STDOUT;    
	i = rpmpldi_init(src, dest, &cb);
	if (i)
		err = 1;

	RESTORE_STDOUT;

	switch (i) {
	case 0:
		log_message(_("RPM base opened"));
		break;

	case 1:
		log_message(_("Can't create dierctory for rpm base"));
		error_msg = _("Could not create directory for RPM base "
			      "on your destantion media.\n"
			      "Please correct this problem and try agin.");
		break;

	case 2:
		log_message(_("RPM base not opened"));
		error_msg = _("Could _not_ open (create) RPM database.\n"
			      "It means that you have some problems with "
			      "destantion drive.\n"
			      "Please check it and try again");
		break;

	default:
		log_message(_("%s default section in switch"), __FUNCTION__);
		error_msg = _("Congratulations"
				"\n\nYou found bug in PLD Installer\n"
				"please don't forget announce it !!!\n"
				"Anyway you probably have some problem with "
				"destantion media.\n Check it and try agin");
		break;
	}

	if (error_msg) 
		goto bail_out;

	REDIRECT_STDOUT;

	if (rpmpldi_scandir()) {
		log_message(_("Can't get list of packages from source media"));
		error_msg = _("Installer could not "
			      "got list of files from your source media.\n"
			      "Please correct it, and try again");
		err = 2;
		goto bail_out;
	}

	if (pkgsListFname)
		rpmpldi_select_from_file(pkgsListFname);
	rpmpldi_selectpackages(names,count);
	RESTORE_STDOUT;

	status_msg(0);

	do {

		if(again){    /* second or higher iteration */
			TPkgSet *pkgs;
			pkgs=rpmpldi_getPkgsSet();
			/* deselect all packages */
			for(i=0;i<pkgs_array_num(pkgs);i++)
				pkgs_array_nth(pkgs,i)->selected=0;
			/* upgrade packages flags an "installed package version"
			 * information */
			rpmpldi_update_installed_info();
		}

		pi_selector();
		
		rpmpldi_add_deps();

		make_progress_window();

		REDIRECT_STDOUT;
		rpmpldi_install(excludeDoc ? RPMTRANS_FLAG_NODOCS : 0);
		RESTORE_STDOUT;

		pi_kill(install_ui.form);
		again = pi_msgbox(pi_add_object(0,
			pi_tag_title, _("Again ?"),
			pi_tag_button_yes, _("_Yes (again)"),
			pi_tag_button_no, _("_No (quit)"),
			pi_tag_caption, _("Do you want install/upgrade "
					  "other packages ?"),
			0));
	} while (again);
	
	sprintf(message,_("Output of all packages' instalation scripts has been"
				" saved in file %s"),stdoutfile);

	pi_msgbox(pi_add_object(0,
		pi_tag_title, _("Important"),
		pi_tag_button_ok, _("_OK, I will check it"),
		pi_tag_caption, message,
	0));
	
	/* stdoutfile  */

bail_out:
	redirect_out(-1);
	status_msg(0);
	if (error_msg)
		pi_msgbox(pi_add_object(0,
			pi_tag_title, _("Error"),
			pi_tag_button_ok, _("_OK"),
			pi_tag_caption, error_msg,
			0));
	rpmpldi_finish();	/* zakonczenie pracy z  rpmmen'em */
	return err;
}

/* module entry point */
void pi_ui(struct module_context *ctx)
{
	if (geteuid() != 0) {
		printf(_("This program is useful only when started "
			 "with root priviliges !!!\n"));
		exit(1);
	}

	pi_init();
	pi_cls();

	pi_set_tag(pi_root_object, pi_tag_text_left_up, _("_Wuch v" VERSION));
	pi_set_tag(pi_root_object, pi_tag_text_right_up, 
		   _("_Polished _Linux _Distribution"));

	rpmmenUI_run("", (char*)ctx->root, NULL, ctx->upgrade_list,
		     ctx->upgrade_count);
		     
	pi_fini();
}
