/*
  Module from RedHat Linux Installer.
  Some minor changes by Pawel Gajda.
  TODO:
   - syslog() support
   - multiple logfiles?
*/

/*
 * Copyright 1998 Red Hat Software 
 *
 * This software may be freely redistributed under the terms of the GNU
 * public license.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 *
 */

/*
  $Id: log.c,v 1.1.1.1 2000/11/05 22:15:06 pawelk Exp $
*/

#include <stdarg.h>
#include <stdio.h>
#include <stdlib.h>
#include <string.h>

#include <unistd.h>
#include <fcntl.h>

#ifdef USE_XMALLOCS
# include "xmalloc.h"
#endif

#include "log.h"

    

static void (*debugMsgFunc)(const char*, ...) = NULL;
static FILE *logfile = NULL;

/* flags */
static int logDebugMessages = 0;
static int logDebugMessagesToScreen = 0;

static void do_log_message(const char *fmt, va_list args);


/*
 */
void log_message(const char *fmt, ...)
{
    va_list args;

    va_start(args, fmt);
    do_log_message(fmt, args);
    va_end(args);

    return;
}

/*
 */
void do_log_debug_message(char *fmt, ...)
{
    va_list args;
    char *tmp = fmt, *s = NULL;
    
    if (!logDebugMessages)
        return;

    /* add 'DEBUG: ' prefix */
    s = malloc(strlen(fmt)+10);
    if(s == NULL) {
        s = fmt;
    } else {
        strcpy(s, "DEBUG: ");
        strcat(s, fmt);
    }
    fmt = s;
    
    va_start(args, fmt); 
    do_log_message(fmt, args);
    
    if (logDebugMessagesToScreen) { /* log to screen too */
        if(debugMsgFunc) {
            char buf[512]; 
            vsprintf(buf, fmt, args);
            debugMsgFunc(buf);
        } else {
            vfprintf(stderr, fmt, args);
        }
    }
    va_end(args);
    
    if (s)
        free(s);
    fmt = tmp;
    return;
}

/*
 */
static void do_log_message(const char *fmt, va_list args)
{
    if (logfile == NULL)
        return;

    fprintf(logfile, "* ");
    vfprintf(logfile, fmt, args);
    fprintf(logfile, "\n");

    fflush(logfile);
}

/*
  open_log
 */
int log_openlog(const char *pathname)
{
    if(logfile != NULL)
        return 0;
    
    logfile = fopen(pathname, "w");

    if (getenv(LOGDEBUG_ENV))
        logDebugMessages = 1;

    if (getenv(LOGDEBUGSCR_ENV))
        logDebugMessagesToScreen = 1;

    return logfile != NULL;
}

/*
  
 */
void log_closelog(void)
{
    if (logfile != NULL) {
	fclose(logfile);
        logfile = NULL;
    }
}


void log_set_callback(void (*msgfunc)(const char *, ...))
{
    debugMsgFunc = msgfunc;
}
