/*
   $Id: wuch.c,v 1.27 2001/10/14 08:44:51 pawelk Exp $
Author: Pawe Koodziej <pawelk@pld.org.pl>
 */
#include <config.h>

#include <sys/types.h>
#include <sys/stat.h>
#include <fcntl.h>
#include <stdio.h>
#include <stdlib.h>
#include <unistd.h>
#include <string.h>
#include <netdb.h>
#include <popt.h>
#include <errno.h>
#include "utils.h"
#include "cfile.h"
#include "wuch.h"
#define MAIN_FILE
#include "module.h"
#define GRADE 1


#ifdef ENABLE_MODULAR
#include <dlfcn.h>

static void *try_load(struct module_context *ctx, 
		      const char *dir, int append_modules)
{
	void *h = 0;
	char path[1024];
	
	/* try both w/ and w/o lib_ */
	
	if (snprintf(path, sizeof(path), "%s%s/mod_%s.so", 
	             dir, append_modules ? "/modules" : "", 
	             ctx->self->name) == -1)
		goto e_path;

	/* if it exists, and has +x,
	 * but for some reason we cannot load it
	 * treat it as fatal
	 */
	if (access(path, X_OK) == 0 &&
	    (h = dlopen(path, RTLD_GLOBAL|RTLD_NOW)) == 0)
	    	goto e_so;
	else {
		if (snprintf(path, sizeof(path), "%s%s/lib_mod_%s.so", 
			     dir, append_modules ? "/modules" : "", 
			     ctx->self->name) == -1)
			goto e_path;
		if (access(path, X_OK) == 0 &&
		    (h = dlopen(path, RTLD_GLOBAL|RTLD_NOW)) == 0)
			goto e_so;
	}
	
	return h;

	e_path:
		/* buffer overflow? not likely ... */
		fprintf(stderr, _("Strange paths...\n"));
		exit(1);
		
	e_so:
		fprintf(stderr, _("Cannot load module: %s (%s)\n"),
			ctx->self->name, dlerror());
		exit(1);
}

void load_module(struct module_context *ctx)
{
	void *h = 0;
	void (*func)(struct module_context *);
	char cwd[1024];
	
	
	if (getcwd(cwd, sizeof(cwd)))
		h = try_load(ctx, cwd, 1);
	if (h == 0)
		h = try_load(ctx, MODULEDIR, 0);
	if (h == 0)
		h = try_load(ctx, "/usr/lib/wuch", 1);
		
	if (h == 0) {
		fprintf(stderr, _("Cannot find module: %s (%s)\n"),
			ctx->self->name, strerror(errno));
		exit(1);
	}

	snprintf(cwd, sizeof(cwd), "%s_ui", ctx->self->name);
	func = dlsym(h, cwd);

	if (func == 0) {
		fprintf(stderr, _("Cannot find symbol '%s' in %s\n"), 
			cwd, ctx->self->name);
		exit(1);
	}

	func(ctx);

	dlclose(h);
}
#endif

void print_summary();

struct ui_module ui_modules[] = {
	{ "newt",  newt_ui,
		gettext_noop("Textmode Newt interface") },
	{ "batch", batch_ui,
		gettext_noop("Textmode non-interactive interface") },
#ifdef ENABLE_STATIC_GRADE 
	{ "grade", grade_ui,
		gettext_noop("Massive Opinion Project mode") },
#endif 
#ifdef ENABLE_MODULAR
	{ "grade", grade_ui,
		gettext_noop("Massive Opinion Project mode") },
	{ "pi", pi_ui,
		gettext_noop("Textmode P user Interface mode (doesn't work)") },
#endif
	{ 0, 0, 0 }
};

/*
 * we use a small trick here, only first character
 * of module name is significant
 */
struct ui_module *find_ui_module(const char *name)
{
	int i;

	for (i = 0; ui_modules[i].name; i++)
		if (ui_modules[i].name[0] == name[0])
			break;
	return (ui_modules[i].name ? &ui_modules[i] : NULL);
}

int main(int ac, const char *av[])
{
	struct module_context ctx;
	char buf[200];
	char *confFile = strdup(CONF_FNAME);
	char *rootDir = 0;
	int rc;
	struct poptOption options[]={
		{"config-file",	'\0',POPT_ARG_STRING,&confFile, 0,NULL,NULL},
		{"destination",	'd' ,POPT_ARG_STRING,&rootDir,  0,NULL,NULL},
		{"source",	'\0',POPT_ARG_STRING,NULL,    103,NULL,NULL},
		{"packages-list-file",'c',POPT_ARG_STRING,&pkgsListFname,
			0,NULL,NULL},
		{"check-deps",'D',POPT_ARG_NONE,NULL,MODE_CHECK_DEPS,
			NULL,NULL},
		{"grade",'g',POPT_ARG_NONE,NULL,MODE_GRADE,NULL,NULL},
		{"batch",'b',POPT_ARG_NONE,NULL,MODE_BATCH,NULL,NULL},
		{"package",'U',POPT_ARG_STRING,NULL,100,NULL,NULL},
		{"module",'m',POPT_ARG_STRING,NULL,101,NULL,NULL},
		{"remove-cache",'k',POPT_ARG_NONE,NULL,102,NULL,NULL},
		{"save-list",'L',POPT_ARG_NONE,NULL,MODE_LIST,NULL,NULL},
	
		POPT_AUTOHELP
		{NULL,0,0,NULL,0}
	};
			
	poptContext optCon;
	
	setlocale(LC_ALL, "");
	bindtextdomain(PACKAGE, LOCALEDIR);
	textdomain(PACKAGE);


	ctx.upgrade_count = 0;
	ctx.upgrade_list = NULL;
	ctx.mode = MODE_VISUAL;
//	ctx.self = &ui_modules[0];
	ctx.self = 0;

	optCon=poptGetContext(NULL,ac,av,options,0);

	/* Check only for --config-file */
	while(poptGetNextOpt(optCon)>=0)
		/* nothing */ ;
	readConfig(confFile,1);
	sprintf(buf,"%s/etc/wuch.conf",getenv("HOME")?getenv("HOME"):"");
	readConfig(buf,0);
	sprintf(buf,"%s/.wuch.conf",getenv("HOME")?getenv("HOME"):"");
	readConfig(buf,0);
	
	poptFreeContext(optCon);
	
	/* check for all other options */
	optCon=poptGetContext(NULL,ac,av,options,0);
	while((rc=poptGetNextOpt(optCon))>=0){
		switch(rc){
			case 102:
				remove_cache++;
				break;
			case 100: 
				ctx.upgrade_list=realloc(ctx.upgrade_list,
					sizeof(char*)*(ctx.upgrade_count+1));
				ctx.upgrade_list[ctx.upgrade_count]=
					strdup(poptGetOptArg(optCon));
				ctx.upgrade_count++;
				break;
			case 101:
				ctx.self = find_ui_module(poptGetOptArg(optCon));
				if (ctx.self == 0) {
					print_summary();
					exit(1);
				}
				break;
			case MODE_CHECK_DEPS:	
				rootDir=BATCH_TMP_DB;
			case MODE_BATCH:
				ctx.self = find_ui_module("batch");
				ctx.mode = rc;
				break;
			case MODE_GRADE:
				ctx.self = find_ui_module("grade");
				ctx.mode = rc;
 				break;
			case MODE_LIST:
				ctx.mode = rc;
				break;
			case 103:
				addSource(poptGetOptArg(optCon));
				break;
		}
	}
					
	if (ctx.self == 0)
		ctx.self = ui_modules + 0;
	if(rc<-1){
		printf(_("Command line parsing error !\n"));
		print_summary();
		exit(1);
	}

				
	if(rootDir)
		setRootDir(rootDir);

	if(!getRootDir()){
		fprintf(stderr,_("Destination directory has to "
			"be specified !!!\n"
			"Use %s -h to get some more info\n"), av[0]);
		exit(1);
	}
	
	sprintf(buf,"%s/debug.log",getRootDir());
	log_open(buf);

	psSetInit();

	ctx.root = getRootDir();
	ctx.self->starter(&ctx);

	log_close();
	
	return 0;
}

void print_summary()
{
	int i;
	char *usage_txt =
	 _(
"Send bug-repoprts to pld-installer@pld.org.pl\n"
"Options:\n"
"  -s source/dir      -- source directory (witchout following /RPMS !)\n"
"  -d destination/dir -- destination directory\n"
"  -c coinst.rpmy     -- file (local) with list of packages to be installed\n"
"  -U packagename     -- select pacakges  \"packagename\" for install/upgrade\n"
"                        this option can by specyfied repatedly\n"
"  -b                 -- batch mode -- not install only print names of\n"
"                        packages (useful witch -U option)\n"
"  -D                 -- print all packages from source media, that require\n"
"                        some feature which is not provided by any other package"
"                        (it's useful only for distribution developers)\n"
"  -m module          -- run specfied module\n"
);
	fprintf(stderr, _("Wuch version %s. "), VERSION);
	fprintf(stderr,"%s", usage_txt);
	fprintf(stderr,_("\nConfiguration file is: %s\n"),CONF_FNAME);

	fprintf(stderr, _("\nAvailable modules:\n"));
	for (i = 0; ui_modules[i].name; i++)
		fprintf(stderr, "%-8s %s\n", ui_modules[i].name,
			_(ui_modules[i].description));
}

